//  Sample Code (Usage) of FIAPUploadAgent
//
//  2011/09/18 ver.1   H.Inoue & H.Ochiai
//
//  ★の部分はセンサ毎に設定（書き換え）が必要な項目

#include <SPI.h>              // Ethernetシールド用
#include <Ethernet.h>         // Ethernetシールド用
#include <FIAPUploadAgent.h>  // FIAPUploadAgent
#include <Time.h>             // 時刻関数

// 全体パラメータ
const unsigned int MEASURE_INTERVAL = 60;  // アップロード間隔(秒)

// ローカル・ネットワーク設定
static byte local_mac[6] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };  // ★ローカル環境に合わせる
static byte local_ip4[4]     = { 192, 168, 11, 55 }; // ★ローカル環境に合わせる
static byte local_gateway[4] = { 192, 168, 11, 1 };  // ★ローカル環境に合わせる
static byte local_netmask[4] = { 255, 255, 255, 0 }; // ★ローカル環境に合わせる

// IEEE1888サーバ設定
static byte server_ip4[4]  = { 133, 11, 168, 97 };                  // ★利用するサーバに合わせる
static char server_host[] = "fiap-sandbox.gutp.ic.i.u-tokyo.ac.jp"; // ★利用するサーバに合わせる
static char server_path[] = "/axis2/services/FIAPStorage";          // ★利用するサーバに合わせる
static int server_port = 80;

// PointSet ID (=PointIDのプレフィックス) の設定
static char fiap_id_prefix[] = "http://gutp.jp/Arduino/demo/"; // ★他と重複しない自分のIDを使うこと

// 値保持フィールド
char str_P1[] = "ON";
char str_P2[] = "OFF";
char timezone[] = "+00:00";  // UTC(GMT)

// 各ポイントの設定
struct fiap_element element[]=
{
  {"P1", str_P1, 0,0,0,0,0,0,timezone},
  {"P2", str_P2, 0,0,0,0,0,0,timezone},
};

// FIAPUploadAgent のインスタンス
FIAPUploadAgent FIAP;

// 初期設定ルーチン
void setup()
{
  Serial.begin(9600);

  // ネットワーク初期化
  Serial.print("IP = "); Serial.println(ip_to_str(local_ip4));
  Serial.print("NM = "); Serial.println(ip_to_str(local_netmask));
  Serial.print("GW = "); Serial.println(ip_to_str(local_gateway));
  Ethernet.begin(local_mac, local_ip4, local_gateway, local_netmask);

  // FIAPライブラリの初期化
  FIAP.begin(server_ip4, server_host, server_path, server_port, fiap_id_prefix);
  
  // 2011年11月10日00時00分00秒(UTC) を 時刻(time)ライブラリに設定 (あくまでサンプル)
  setTime(1320818400);
}

// メインループ
void loop()
{
  int i;
  time_t t;

  // 各データに現在時刻をセット
  t = now();
  for(i = 0; i < 2; i++) {
     element[i].year = year(t);
     element[i].month = month(t);
     element[i].day = day(t);
     element[i].hour = hour(t);
     element[i].minute = minute(t);
     element[i].second = second(t);
  }

  // データ設定(文字列として格納)
  sprintf(str_P1, "ON");
  sprintf(str_P2, "OFF");

  // IEEE1888アップロードを実行
  Serial.print("FIAP uploading... ");
  int ret = FIAP.post(element, 2);
  Serial.print("status="); Serial.println(ret);

  // 測定間隔だけ待つ
  delay(MEASURE_INTERVAL * 1000UL);
}

// IPアドレス(IPv4)をドットで区切られた文字列に変換する
char *ip_to_str(byte *ip)
{
  static char str[16];
  sprintf(str, "%d.%d.%d.%d\0", ip[0], ip[1], ip[2], ip[3]);
  return(str);
}

// end of code
